package sun.nio.fs;

import java.util.regex.PatternSyntaxException;

public class Globs
{
    private static final String regexMetaChars = ".^$+{[]|()";
    private static final String globMetaChars = "\\*?[{";
    private static char EOL;
    
    private Globs() {
    }
    
    private static boolean isRegexMeta(char ch) {
        return ".^$+{[]|()".indexOf((int)ch) != -1;
    }
    
    private static boolean isGlobMeta(char ch) {
        return "\\*?[{".indexOf((int)ch) != -1;
    }
    
    private static char next(String s, int index) {
        if (index < s.length()) {
            return s.charAt(index);
        }
        return Globs.EOL;
    }
    
    private static String toRegexPattern(String regex, boolean b) {
        int n = 0;
        StringBuilder sb = new StringBuilder("^");
        int i = 0;
        while (i < regex.length()) {
            int n2 = regex.charAt(i++);
            switch (n2) {
                case 92: {
                    if (i == regex.length()) {
                        throw new PatternSyntaxException("No character to escape", regex, i - 1);
                    }
                    int char1 = regex.charAt(i++);
                    if (isGlobMeta((char)char1) || isRegexMeta((char)char1)) {
                        sb.append('\\');
                    }
                    sb.append((char)char1);
                    continue;
                }
                case 47: {
                    if (b) {
                        sb.append("\\\\");
                        continue;
                    }
                    sb.append((char)n2);
                    continue;
                }
                case 91: {
                    if (b) {
                        sb.append("[[^\\\\]&&[");
                    }
                    else {
                        sb.append("[[^/]&&[");
                    }
                    if (next(regex, i) == '^') {
                        sb.append("\\^");
                        ++i;
                    }
                    else {
                        if (next(regex, i) == '!') {
                            sb.append('^');
                            ++i;
                        }
                        if (next(regex, i) == '-') {
                            sb.append('-');
                            ++i;
                        }
                    }
                    int n3 = 0;
                    int n4 = '\0';
                    while (i < regex.length()) {
                        n2 = regex.charAt(i++);
                        if (n2 == ']') {
                            break;
                        }
                        if (n2 == '/' || (b && n2 == '\\')) {
                            throw new PatternSyntaxException("Explicit 'name separator' in class", regex, i - 1);
                        }
                        if (n2 == '\\' || n2 == '[' || (n2 == '&' && next(regex, i) == '&')) {
                            sb.append('\\');
                        }
                        sb.append((char)n2);
                        if (n2 == '-') {
                            if (n3 == 0) {
                                throw new PatternSyntaxException("Invalid range", regex, i - 1);
                            }
                            if ((n2 = next(regex, i++)) != Globs.EOL) {
                                if (n2 == ']') {
                                    break;
                                }
                                if (n2 < n4) {
                                    throw new PatternSyntaxException("Invalid range", regex, i - 3);
                                }
                                sb.append((char)n2);
                                n3 = 0;
                            }
                            else {
                                break;
                            }
                        }
                        else {
                            n3 = 1;
                            n4 = n2;
                        }
                    }
                    if (n2 != ']') {
                        throw new PatternSyntaxException("Missing ']", regex, i - 1);
                    }
                    sb.append("]]");
                    continue;
                }
                case 123: {
                    if (n != 0) {
                        throw new PatternSyntaxException("Cannot nest groups", regex, i - 1);
                    }
                    sb.append("(?:(?:");
                    n = 1;
                    continue;
                }
                case 125: {
                    if (n != 0) {
                        sb.append("))");
                        n = 0;
                        continue;
                    }
                    sb.append('}');
                    continue;
                }
                case 44: {
                    if (n != 0) {
                        sb.append(")|(?:");
                        continue;
                    }
                    sb.append(',');
                    continue;
                }
                case 42: {
                    if (next(regex, i) == '*') {
                        sb.append(".*");
                        ++i;
                        continue;
                    }
                    if (b) {
                        sb.append("[^\\\\]*");
                        continue;
                    }
                    sb.append("[^/]*");
                    continue;
                }
                case 63: {
                    if (b) {
                        sb.append("[^\\\\]");
                        continue;
                    }
                    sb.append("[^/]");
                    continue;
                }
                default: {
                    if (isRegexMeta((char)n2)) {
                        sb.append('\\');
                    }
                    sb.append((char)n2);
                    continue;
                }
            }
        }
        if (n != 0) {
            throw new PatternSyntaxException("Missing '}", regex, i - 1);
        }
        return sb.append('$').toString();
    }
    
    static String toUnixRegexPattern(String s) {
        return toRegexPattern(s, false);
    }
    
    static String toWindowsRegexPattern(String s) {
        return toRegexPattern(s, true);
    }
    
    static {
        Globs.EOL = '\0';
    }
}
